#!/usr/bin/env node
import { Clerc, Root, defineCommand, friendlyErrorPlugin, helpPlugin, notFoundPlugin, strictFlagsPlugin } from "clerc";
import { access, copyFile, mkdir, readdir, rm, stat, writeFile } from "node:fs/promises";
import path, { basename, dirname, join, resolve } from "node:path";
import process from "node:process";
import { cancel, confirm, intro, isCancel, log, outro, select, spinner, text } from "@clack/prompts";
import { bold, green } from "yoctocolors";
import { execa, execaSync } from "execa";
import { fileURLToPath } from "node:url";
import fs from "node:fs";

//#region package.json
var name = "create-vue-vine";
var version = "1.0.0";
var description = "Official CLI for creating Vue Vine project.";

//#endregion
//#region src/utils/banner.ts
/**
* generated by the following code:
*
* import gradient from 'gradient-string'
*
* gradient(['#364fae', '#43954b'])
*         ('Vue Vine - Another style of writing Vue components')
*
* Use the output directly here to keep the bundle small.
*
* How to generate:
*
* - get graient text
* - text.replace(/\x1B/g, '\\x1B')
* - write the output to a txt file (which will not handle the ansi escapes) and copy the output here
*/
const gradientBanner = "\x1B[38;2;54;79;174mV\x1B[39m\x1B[38;2;54;81;172mu\x1B[39m\x1B[38;2;55;82;169me\x1B[39m \x1B[38;2;55;84;167mV\x1B[39m\x1B[38;2;55;86;164mi\x1B[39m\x1B[38;2;56;88;162mn\x1B[39m\x1B[38;2;56;89;160me\x1B[39m \x1B[38;2;56;91;157m-\x1B[39m \x1B[38;2;57;93;155mA\x1B[39m\x1B[38;2;57;94;152mn\x1B[39m\x1B[38;2;57;96;150mo\x1B[39m\x1B[38;2;57;98;147mt\x1B[39m\x1B[38;2;58;99;145mh\x1B[39m\x1B[38;2;58;101;143me\x1B[39m\x1B[38;2;58;103;140mr\x1B[39m \x1B[38;2;59;105;138ms\x1B[39m\x1B[38;2;59;106;135mt\x1B[39m\x1B[38;2;59;108;133my\x1B[39m\x1B[38;2;60;110;131ml\x1B[39m\x1B[38;2;60;111;128me\x1B[39m \x1B[38;2;60;113;126mo\x1B[39m\x1B[38;2;61;115;123mf\x1B[39m \x1B[38;2;61;117;121mw\x1B[39m\x1B[38;2;61;118;118mr\x1B[39m\x1B[38;2;62;120;116mi\x1B[39m\x1B[38;2;62;122;114mt\x1B[39m\x1B[38;2;62;123;111mi\x1B[39m\x1B[38;2;63;125;109mn\x1B[39m\x1B[38;2;63;127;106mg\x1B[39m \x1B[38;2;63;129;104mV\x1B[39m\x1B[38;2;64;130;102mu\x1B[39m\x1B[38;2;64;132;99me\x1B[39m \x1B[38;2;64;134;97mc\x1B[39m\x1B[38;2;64;135;94mo\x1B[39m\x1B[38;2;65;137;92mm\x1B[39m\x1B[38;2;65;139;89mp\x1B[39m\x1B[38;2;65;140;87mo\x1B[39m\x1B[38;2;66;142;85mn\x1B[39m\x1B[38;2;66;144;82me\x1B[39m\x1B[38;2;66;146;80mn\x1B[39m\x1B[38;2;67;147;77mt\x1B[39m\x1B[38;2;67;149;75ms\x1B[39m";

//#endregion
//#region src/utils/clack.ts
function cancel$1(...args) {
	cancel(...args);
	process.exit(0);
}
function wrapClack(fn) {
	return async (...args) => {
		const result = await fn(...args);
		if (isCancel(result)) {
			cancel$1("Operation cancelled. Goodbye!");
			process.exit(0);
		}
		return result;
	};
}
const text$1 = wrapClack(text);
const select$1 = wrapClack(select);
const confirm$1 = wrapClack(confirm);

//#endregion
//#region src/utils/flags.ts
function defineFlagMeta(flag) {
	return flag;
}

//#endregion
//#region src/utils/fs.ts
function exists(path$1) {
	return access(path$1).then(() => true).catch(() => false);
}

//#endregion
//#region src/utils/pm.ts
const userAgentEnv = process.env.npm_config_user_agent ?? "";
function detectPackageManager() {
	const [fromUserAgent] = userAgentEnv.match(/pnpm|yarn|bun/) || [];
	if (fromUserAgent) return fromUserAgent;
	let pnpmVersionExitCode = -1;
	let yarnVersionExitCode = -1;
	try {
		pnpmVersionExitCode = execaSync("pnpm", ["--version"], { stdio: "ignore" }).exitCode;
	} catch {}
	try {
		yarnVersionExitCode = execaSync("yarn", ["--version"], { stdio: "ignore" }).exitCode;
	} catch {}
	if (pnpmVersionExitCode === 0) return "pnpm";
	else if (yarnVersionExitCode === 0) return "yarn";
	return "npm";
}
const packageManager = detectPackageManager();
function getPmCommand(command) {
	let commands;
	if (packageManager === "pnpm") commands = ["pnpm", [command]];
	else commands = [packageManager, [command === "install" ? "" : "run", command].filter(Boolean)];
	return commands;
}
function formatPmCommand(commands) {
	return commands.flat().join(" ");
}
function runPmCommand(command, cwd) {
	return execa(...getPmCommand(command), {
		stdio: "inherit",
		cwd
	});
}

//#endregion
//#region src/utils/project.ts
function validateProjectName(path$1) {
	return basename(path$1) === path$1;
}
const __filename = fileURLToPath(import.meta.url);
const __dirname = dirname(__filename);
async function getTemplateDirectory() {
	const templateRoot = resolve(__dirname, "..", "template");
	return await exists(templateRoot) ? templateRoot : void 0;
}

//#endregion
//#region src/utils/deepMerge.ts
const isObject = (val) => val && typeof val === "object";
const mergeArrayWithDedupe = (a, b) => Array.from(new Set([...a, ...b]));
/**
* Recursively merge the content of the new object to the existing one
* @param target the existing object
* @param obj the new object
*/
function deepMerge(target, obj) {
	for (const key of Object.keys(obj)) {
		const oldVal = target[key];
		const newVal = obj[key];
		if (Array.isArray(oldVal) && Array.isArray(newVal)) target[key] = mergeArrayWithDedupe(oldVal, newVal);
		else if (isObject(oldVal) && isObject(newVal)) target[key] = deepMerge(oldVal, newVal);
		else target[key] = newVal;
	}
	return target;
}
var deepMerge_default = deepMerge;

//#endregion
//#region src/utils/sortDependencies.ts
function sortDependencies(packageJson) {
	const sorted = {};
	const depTypes = [
		"dependencies",
		"devDependencies",
		"peerDependencies",
		"optionalDependencies"
	];
	for (const depType of depTypes) if (packageJson[depType]) {
		sorted[depType] = {};
		Object.keys(packageJson[depType]).sort().forEach((name$1) => {
			sorted[depType][name$1] = packageJson[depType][name$1];
		});
	}
	return {
		...packageJson,
		...sorted
	};
}

//#endregion
//#region src/utils/renderTemplate.ts
async function renderTemplate(src, dest) {
	const stats = await stat(src);
	if (stats.isDirectory()) {
		await mkdir(dest, { recursive: true });
		const files = await readdir(src);
		await Promise.all(files.map((file) => renderTemplate(path.resolve(src, file), path.resolve(dest, file))));
		return;
	}
	const filename = path.basename(src);
	if (filename.startsWith("_")) dest = path.resolve(path.dirname(dest), `.${filename.slice(1)}`);
	if (filename === "package.json" && fs.existsSync(dest)) {
		const existing = JSON.parse(fs.readFileSync(dest, "utf8"));
		const newPackage = JSON.parse(fs.readFileSync(src, "utf8"));
		const pkg = sortDependencies(deepMerge_default(existing, newPackage));
		if (src.endsWith("common/package.json")) await setLatestVineVersion(pkg, "vue-vine");
		else if (src.endsWith("ts/package.json")) await setLatestVineVersion(pkg, "vue-vine-tsc");
		fs.writeFileSync(dest, `${JSON.stringify(pkg, null, 2)}\n`);
		return;
	}
	await copyFile(src, dest);
}
let isLastFetchTimeout = false;
async function getLatestVineVersion(dep) {
	if (isLastFetchTimeout) return;
	const controller = new AbortController();
	const timeout = setTimeout(() => {
		isLastFetchTimeout = true;
		controller.abort();
	}, 3e3);
	try {
		const res = await fetch(`https://npm.antfu.dev/${dep}@latest`, { signal: controller.signal });
		const pkg = await res.json();
		return pkg.version;
	} finally {
		clearTimeout(timeout);
	}
}
async function setLatestVineVersion(pkg, dep) {
	try {
		const latestVersion = await getLatestVineVersion(dep);
		if (!latestVersion) return;
		pkg.devDependencies[dep] = `^${latestVersion}`;
	} catch {}
}

//#endregion
//#region src/create.ts
const builtInTemplates = [
	"common",
	"code/base",
	"config/ts",
	"config/eslint"
];
function createProjectOptions(params) {
	return {
		...params,
		templates: []
	};
}
async function createProject(options) {
	const templateDirectory = await getTemplateDirectory();
	const withBase = (path$1) => join(templateDirectory, path$1);
	await mkdir(options.path, { recursive: true });
	const splitName = options.name.startsWith("@") ? options.name : options.name.replace("/", "-").replace(/^[./]+|-+$/, "");
	await writeFile(join(options.path, "package.json"), JSON.stringify({ name: splitName }, null, 2));
	for (const template of [...builtInTemplates, ...options.templates]) await renderTemplate(withBase(template), options.path);
}

//#endregion
//#region src/flags/index.ts
const metas = {
	router: defineFlagMeta({
		name: "router",
		message: "Use Vue Router?",
		flag: {
			type: Boolean,
			description: "Add Vue Router",
			default: true
		},
		initialValue: true
	}),
	store: defineFlagMeta({
		name: "store",
		message: "Use Pinia as state management?",
		flag: {
			type: Boolean,
			description: "Add Pinia",
			default: true
		},
		initialValue: true
	}),
	install: defineFlagMeta({
		name: "install",
		message: "Install all dependencies for the project now?",
		flag: {
			type: Boolean,
			description: "Install dependencies",
			alias: "i",
			default: false
		},
		initialValue: false
	})
};
const flags$1 = Object.entries(metas).reduce((acc, [key, value]) => {
	Reflect.set(acc, key, value.flag);
	return acc;
}, {});
function useFlags() {
	return {
		flags: flags$1,
		executeFlags: async (flags$2, options) => {
			const context = { template: (...path$1) => {
				options.templates.push(...path$1);
			} };
			const css = await select$1({
				message: "Using atomized css?",
				options: [
					{
						value: "unocss",
						label: "UnoCSS"
					},
					{
						value: "tailwind",
						label: "Tailwind"
					},
					{
						value: "no",
						label: "No"
					}
				]
			});
			for (const key in metas) {
				const metaKey = key;
				const { initialValue, message } = metas[metaKey];
				flags$2[metaKey] = await confirm$1({
					message,
					initialValue
				});
			}
			if (flags$2.router) context.template("code/router", "config/router");
			if (flags$2.store) {
				context.template("code/store/common", "config/pinia");
				if (flags$2.router) context.template("code/store/with-router");
				else context.template("code/store/base");
			}
			if (css !== "no") context.template(`code/css/${css}`);
			else context.template("code/css/base");
		}
	};
}

//#endregion
//#region src/commands/create.ts
const defaultProjectName = "vue-vine-project";
const { flags, executeFlags } = useFlags();
const createCommand = defineCommand({
	name: Root,
	description: "Create a Vue Vine project",
	parameters: ["[projectName]"],
	flags: {
		force: {
			type: Boolean,
			description: "Delete existing folder",
			alias: "f",
			default: false
		},
		...flags
	},
	alias: "create"
}, async (ctx) => {
	intro(gradientBanner);
	const templateDir = await getTemplateDirectory();
	if (!templateDir) cancel$1("Unable to find template directory");
	let projectName = ctx.parameters.projectName || defaultProjectName;
	if (!ctx.parameters.projectName) projectName = await text$1({
		message: "Project name:",
		placeholder: defaultProjectName,
		defaultValue: defaultProjectName,
		validate: (value) => {
			if (!validateProjectName(value)) return "Invalid project name";
		}
	});
	const cwd = process.cwd();
	const projectPath = join(cwd, projectName);
	const isSameProjectExists = await exists(projectPath);
	if (isSameProjectExists) {
		if (!ctx.flags.force) ctx.flags.force = await confirm$1({
			message: `Folder ${projectName} already exists. Delete?`,
			initialValue: false
		});
		if (!ctx.flags.force) cancel$1(`Folder ${projectName} already exists. Goodbye!`);
		else {
			log.info(`Folder ${projectName} will be deleted.`);
			await rm(projectPath, { recursive: true });
		}
	}
	const projectOptions = createProjectOptions({
		path: projectPath,
		name: projectName,
		templateDir
	});
	await executeFlags(ctx.flags, projectOptions);
	const s = spinner();
	s.start(`Creating project ${projectName}`);
	await createProject(projectOptions);
	s.stop(`Project created at: ${projectPath}`);
	if (ctx.flags.install) {
		s.start("Installing dependencies");
		await runPmCommand("install", projectPath);
		s.stop("Dependencies installed!");
	}
	const helpText = [
		"You're all set! Now run:",
		"",
		`   cd ${bold(green(projectName.includes(" ") ? `"${projectName}"` : projectName))}`,
		ctx.flags.install ? void 0 : `   ${bold(green(formatPmCommand(getPmCommand("install"))))}`,
		`   ${bold(green(formatPmCommand(getPmCommand("dev"))))}`,
		"",
		"   Happy hacking!"
	].filter((s$1) => s$1 !== void 0).join("\n");
	outro(helpText);
	process.exit();
});

//#endregion
//#region src/index.ts
Clerc.create(name, version, description).use(helpPlugin()).use(notFoundPlugin()).use(strictFlagsPlugin()).use(notFoundPlugin()).use(friendlyErrorPlugin()).command(createCommand).parse();

//#endregion